package com.nationrel.modules.weibull.common.excel.logic;

import com.nationrel.modules.weibull.common.excel.app.ExcelApp;
import com.nationrel.modules.weibull.common.excel.bean.ExcelAppBean;
import com.nationrel.modules.weibull.common.excel.util.ExcelAppUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.util.SpringContextUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.*;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

/**
 * @ClassName ExcelAppFactory
 * @Description Excel表格处理工厂
 * @version V1.0
 */
public class ExcelAppFactory {

    /** 缓存 */
    private static Map<String, List<? extends Object>> cache;
//    /** 配置上下文 */
//    private static ApplicationContext ac;

    /**
     * <p>Title: </p>
     * <p>Description: </p>
     */
    private ExcelAppFactory() {
        init();
    }
    /**
     * @ClassName ExcelAppFactoryHolder
     * @Description 实例获得
     * @Author sam
     * @CreateDate 2015-7-21
     * @version V1.0
     */
    private static class ExcelAppFactoryHolder {
        private static ExcelAppFactory instance = new ExcelAppFactory();
    }
    /**
     * @Title: getInstance
     * @Description: 获得excel处理工厂
     * @return ExcelAppFactory
     */
    public static ExcelAppFactory getInstance() {
        return ExcelAppFactoryHolder.instance;
    }
    /**
     * @Title: init
     * @Description: 初始化Excel处理工厂上下文
     */
    private void init () {
//        if (ac == null) {
//            ac = new ClassPathXmlApplicationContext("classpath:excelAppConfig.xml");
//        }
        if (cache == null) {
            cache = new Hashtable<String, List<? extends Object>>();
        }
    }
    /**
     * @Title: read
     * @Description: Excel处理入口, 如果Cache有， 读取Cache
     * @param beanId 来自于excelAppConfig配置文件
     * @param fileName 源文件名, 可以是全路径
     * @param sheetName 要读取的Excel sheet页面名
     * @return Object 读取到的实体对象
     * @throws Exception e
     */
    public List<? extends Object> readFromCache(String beanId, String fileName, String sheetName) throws Exception {
        List<? extends Object> result = null;
        if (cache.containsKey(beanId)) {
            result = cache.get(beanId);
        } else {
            result = read(beanId, fileName, sheetName);
            cache.put(beanId, result);
        }
        return result;
    }
    /**
     * @Title: handler
     * @Description: Excel处理入口
     * @param beanId 要处理的对象，配置在excepAppConfig.xml里面的beanName
     * @param fileName 源文件名
     * @param sheetName 要读取的sheet页名
     * @return List<Object> 实体对象集合
     * @throws Exception e
     */
    public List<? extends Object> read(String beanId, String fileName, String sheetName) throws Exception {
        List<? extends Object> result = null;
        ExcelAppBean bean = (ExcelAppBean) SpringContextUtil.getBean(beanId);

        // XML文件配置优先
        if (ValidateUtil.isEmpty(bean.getSourceFileName())) {
            bean.setSourceFileName(fileName);
        }
        if (ValidateUtil.isEmpty(bean.getSourceSheetName())) {
            bean.setSourceSheetName(sheetName);
        }

        String suffix = fileName.substring(fileName.lastIndexOf('.') + 1);
        ExcelApp app = (ExcelApp) SpringContextUtil.getBean(suffix.toUpperCase());

        boolean allRow = bean.isAllRow();
        /**
         * 有三种情况:
         * 1. 给定了起始列，然后以起始列为表示向后成块的读取，结束列为所配置的list里面的最后一个
         * 2. 如果没有给定起始列，但是给定了要读取的列号，并且要求读取所配置的这几列的所有行, 例如 4， 标示读取第四列的所有行
         * 3. 无序的读取，给定了行号和列号的配置信息, 例如3,2 表示读取第三行的第二列
         */
        if (bean.getColStartFlag() != null && bean.getColStartFlag().length() > 0) {
            result = app.readByColumnCof(bean);
        } else if (allRow) {
            result = app.readAllRow(bean);
        } else {
            result = app.readByRowNum(bean);
        }
        // 如果需要缓存
        if (bean.isNeedCache()) {
            cache.put(beanId, result);
        }
        return result;
    }
    /**
     * @Title: write
     * @Description: 写Excel, 写到指定目录
     * @param obj obj 要写入的对象
     * @param beanId 来自excelAppConfig.xml文件的beanName
     * @param sourceFileName 源模板文件
     * @param sourceSheetName 源模板的sheet页名
     * @param targetFileName 目标文件
     * @param targetSheetName 目标文件的Sheet名
     * @throws Exception e
     */
    public void write(Object obj, String beanId, String sourceFileName, String sourceSheetName, String targetFileName, String targetSheetName) throws Exception {
        ExcelAppBean bean = (ExcelAppBean) SpringContextUtil.getBean(beanId);

        String suffix = sourceFileName.substring(sourceFileName.lastIndexOf(".") + 1);
        ExcelApp xlsx = (ExcelApp) SpringContextUtil.getBean(suffix.toUpperCase());

        bean.setSourceFileName(sourceFileName);
        bean.setSourceSheetName(sourceSheetName);
        bean.setTargetFileName(targetFileName);
        bean.setTargetSheetName(targetSheetName);
        /**
         * 目前支持到按单元格写，前提条件是给定了一个已知的模板文件. 例如，给定了3,2，表示要写入第三行第二列;给定了3-15,2表示要写3-15行的第二列
         * 后期将扩展写的功能，根据表头，写所有行等
         */
        xlsx.write(obj, bean);
    }
    
    /**
     * @Title: writeExcel
     * @Description: 写Excel, 写到指定目录
     * @param templateFileName 模板文件名称
     * @param isDisplayGridlines 是否显示网格
     * @param startRow 从表格第几行开始写入数据
     * @param dataMap 写入的数据
     * @param classNames 写入数据返回查询对象（完整类路径）
     * @throws Exception e
     */
    public ByteArrayOutputStream writeExcel(String templateFileName,
                           Boolean isDisplayGridlines,
                           int startRow,
                           Map<String,List<Object>> dataMap,
                           String[] classNames) throws Exception {
        
        InputStream is = new FileInputStream(new File(templateFileName));
        XSSFWorkbook xssfwb = new XSSFWorkbook(is);
        Workbook wb = new SXSSFWorkbook(xssfwb, EwbConstants.SXSSF_WINDOW_SIZE);

        // 多个工作表
        for (int i= 0;i<wb.getNumberOfSheets();i++) {
        
            // 得到工作表
            Sheet sh = wb.getSheetAt(i);
            
            String className = classNames[i];
            
            sh.setDisplayGridlines(isDisplayGridlines);
            
            Row row = null;
            Cell cell = null;
            
            // 获取每一个sheet页中将要写入的数据
            List<Object> listObj = dataMap.get("sheet"+i);
            
            int rowCount = startRow;

            if (listObj != null && listObj.size() > 0) {

                // 遍历多行数据 
                for (Object obj : listObj) {
                    
                    row = sh.createRow(rowCount);
                    int cellCount = 0;
                    
                    // 得到类对象
                    Class cla = (Class)obj.getClass();
                    
                    // 得到类对象中所有属性集合
                    Field[] fields = cla.getDeclaredFields();
                    
                    // 遍历每行中多个单元格数据
                    for (int j = 0;j < fields.length;j++) {
                        Field field = fields[j];
                        // 设置所有属性可以访问
                        field.setAccessible(true);
                        
                        // 得到obj对象中属性的属性名称
                        String objName = field.getName();
                        
                        Class<?> c = Class.forName(className);

                        Method    method = c.getDeclaredMethod(ExcelAppUtil.becomeGetMethod(objName));

                        Object objValue = method.invoke(obj);
                        
                        cell = row.createCell(cellCount);
                        
                        if (objValue instanceof Boolean) {
                            cell.setCellValue((Boolean) objValue);
                        } else if (objValue instanceof Date) {
                            cell.setCellValue(ExcelAppUtil.getDateStr((Date) objValue));
                        } else if (objValue instanceof Double) {
                            cell.setCellValue((Double) objValue);
                        } else {
                            if (objValue == null || "".equals(objValue)) {
                                cell.setCellValue("");
                            } else {
                                cell.setCellValue(objValue.toString());
                            }
                        }
                        
                        cellCount++;
                    }
                    
                    rowCount++;
                }
            }
        
        }
        //////////////////////////////////////////////////////
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        wb.write(out);
        out.flush();
        out.close();
        return out;
    }
    
    /**
     * @Title: writeExcel
     * @Description: 写Excel, 写到指定目录
     * @param templateFileName 模板文件名称
     * @param isDisplayGridlines 是否显示网格
     * @param startRow 从表格第几行开始写入数据
     * @param dataMap 写入的数据
     * @param classNames 写入数据返回查询对象（完整类路径）
     * @throws Exception e
     */
    public FileOutputStream writeExcel(File xlsxFile,String templateFileName,
                           Boolean isDisplayGridlines,
                           int startRow,
                           Map<String,List<Object>> dataMap,
                           String[] classNames) throws Exception {
        
        InputStream is = new FileInputStream(new File(templateFileName));
        XSSFWorkbook xssfwb = new XSSFWorkbook(is);
        Workbook wb = new SXSSFWorkbook(xssfwb, EwbConstants.SXSSF_WINDOW_SIZE);

        // 多个工作表
        for (int i= 0;i<wb.getNumberOfSheets();i++) {
        
            // 得到工作表
            Sheet sh = wb.getSheetAt(i);
            
            String className = classNames[i];
            
            sh.setDisplayGridlines(isDisplayGridlines);
            
            Row row = null;
            Cell cell = null;
            
            // 获取每一个sheet页中将要写入的数据
            List<Object> listObj = dataMap.get("sheet"+i);
            
            int rowCount = startRow;

            if (listObj != null && listObj.size() > 0) {

                // 遍历多行数据 
                for (Object obj : listObj) {
                    
                    row = sh.createRow(rowCount);
                    int cellCount = 0;
                    
                    // 得到类对象
                    Class cla = (Class)obj.getClass();
                    
                    // 得到类对象中所有属性集合
                    Field[] fields = cla.getDeclaredFields();
                    
                    // 遍历每行中多个单元格数据
                    for (int j = 0;j < fields.length;j++) {
                        Field field = fields[j];
                        // 设置所有属性可以访问
                        field.setAccessible(true);
                        
                        // 得到obj对象中属性的属性名称
                        String objName = field.getName();
                        
                        Class<?> c = Class.forName(className);

                        Method    method = c.getDeclaredMethod(ExcelAppUtil.becomeGetMethod(objName));

                        Object objValue = method.invoke(obj);
                        
                        cell = row.createCell(cellCount);
                        
                        if (objValue instanceof Boolean) {
                            cell.setCellValue((Boolean) objValue);
                        } else if (objValue instanceof Date) {
                            cell.setCellValue(ExcelAppUtil.getDateStr((Date) objValue));
                        } else if (objValue instanceof Double) {
                            cell.setCellValue((Double) objValue);
                        } else {
                            if (objValue == null || "".equals(objValue)) {
                                cell.setCellValue("");
                            } else {
                                cell.setCellValue(objValue.toString());
                            }
                        }
                        
                        cellCount++;
                    }
                    
                    rowCount++;
                }
            }
        
        }
        //////////////////////////////////////////////////////
        //ByteArrayOutputStream out = new ByteArrayOutputStream();
        FileOutputStream fileOutputStream = new FileOutputStream(xlsxFile);
        wb.write(fileOutputStream);
        fileOutputStream.flush();
        fileOutputStream.close();
        return fileOutputStream;
    }
    
    /**
     * @Title: download
     * @Description: 写Excel, 并以流的方式下载
     * @param obj obj 要写入的对象
     * @param beanId 来自excelAppConfig.xml文件的beanName
     * @param sourceFileName 源模板文件
     * @param sourceSheetName 源模板的sheet页名
     * @param targetFileName 目标文件
     * @param targetSheetName 目标文件的Sheet名
     * @return ByteArrayOutputStream
     * @throws Exception e
     */
    public ByteArrayOutputStream download(Object obj, String beanId, String sourceFileName, String sourceSheetName, String targetFileName, String targetSheetName) throws Exception {

        ExcelAppBean bean = (ExcelAppBean) SpringContextUtil.getBean(beanId);

        String suffix = sourceFileName.substring(sourceFileName.lastIndexOf(".") + 1);
        ExcelApp xlsx = (ExcelApp) SpringContextUtil.getBean(suffix.toUpperCase());

        bean.setSourceFileName(sourceFileName);
        bean.setSourceSheetName(sourceSheetName);
        bean.setTargetFileName(targetFileName);
        bean.setTargetSheetName(targetSheetName);
        /**
         * 目前支持到按单元格写，前提条件是给定了一个已知的模板文件. 例如，给定了3,2，表示要写入第三行第二列;给定了3-15,2表示要写3-15行的第二列
         * 后期将扩展写的功能，根据表头，写所有行等
         * 以流的方式下载
         */
        ByteArrayOutputStream out = xlsx.download(obj, bean);
        return out;
    }
    /**
     * @Title: removeFromCache
     * @Description: 从Cache移除
     * @param key void
     */
    public void  removeFromCache (String key) {
        if (cache != null && cache.containsKey(key)) {
            cache.remove(key);
        }
    }
    /**
     * @Title: deleteFile
     * @Description: 批量删除文件
     * @param filePath 文件路径
     * @param fileName 文件名
     * @param delDir 删除目录
     */
    public void batchDelFile (String filePath, final String fileName, boolean delDir) {
        File file = new File(filePath);
        if (file.isDirectory()) {
            File[] files = file.listFiles(new FileFilter() {
                @Override
                public boolean accept(File file) {
                    if (file.getName().startsWith(fileName)) {
                        return true;
                    }
                    return false;
                }
            });
            // 删除匹配的文件
            for (File result : files) {
                if (result.isFile()) {
                    result.delete();
                }
            }
            // 如果需要删除目录
            if (delDir) {
                file.delete();
            }
        }
    }
    /**
     * @Title: delFile
     * @Description: 删除文件
     * @param fileName 文件名
     */
    public void delFile (String fileName) {
        File file = new File(fileName);
        if (file.isFile()) {
            file.delete();
        }
    }
}
