package com.nationrel.modules.weibull.util;

import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.util.CollectionUtils;

import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 导出Excel
 */
@Slf4j
public class NevadaExportExcelUtil {

    /**
     * 导出Excel
     *
     * @param lists     要导出的数据集合
     * @param colHeader 要导出的excel表头
     * @param response  使用response可以导出到浏览器
     * @return
     */
    public static <T> void export(HttpServletResponse response,
                                  List<String> colHeader,
                                  List<String>... lists) {
        // 设置默认文件名为当前时间：年月日时分秒
        String sheetName = new SimpleDateFormat("yyyyMMddhhmmss").format(new Date());
        // 设置response头信息
        response.reset();
        response.setContentType("application/vnd.ms-excel"); // 改成输出excel文件
        try {
            response.setHeader("Content-disposition", "attachment; filename=" + new String(sheetName.getBytes("gb2312"), EwbConstants.CODING_FORMAT) + EwbConstants.FILE_TYPE_XLSX);
        } catch (UnsupportedEncodingException e1) {
            log.error("{ExportToExcelUtil.export} ==> 导出Excel失败,不支持字符编码");
        }

        try {
            //创建一个WorkBook,对应一个Excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            for (int i = 0; i < lists.length; i++) {
                List<String> list = lists[i];
                if (!CollectionUtils.isEmpty(list)) {
                    //在Workbook中，创建一个sheet，对应Excel中的工作薄（sheet）
                    if (i == 0) {
                        sheetName = EwbConstants.NEVADA_DATA_NS_NAME_ZH;
                    } else if (i == 1) {
                        sheetName = EwbConstants.NEVADA_DATA_UP_NAME_ZH;
                    } else if (i == 2) {
                        sheetName = EwbConstants.NEVADA_DATA_DOWN_NAME_ZH;
                    }
                    XSSFSheet sheet = wb.createSheet(sheetName);
                    sheet.setHorizontallyCenter(true);
                    sheet.setDefaultRowHeight((short) 500);
                    //冻结第一行
                    sheet.createFreezePane(0, 1, 0, 1);
                    //创建单元格，并设置值表头 设置表头居中
                    XSSFCellStyle titleCellStyle = wb.createCellStyle();
                    XSSFCellStyle cellStyle = wb.createCellStyle();
                    cellStyle.setAlignment(HorizontalAlignment.CENTER);
                    cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
                    // 设置：表头Cell样式
                    writeCellStyle(titleCellStyle, FillPatternType.SOLID_FOREGROUND, IndexedColors.LIME.index, HorizontalAlignment.CENTER, VerticalAlignment.CENTER);
                    // 填充工作表
                    fillSheet(sheet, list, colHeader, titleCellStyle, cellStyle);
                }
            }

            //将文件输出
            OutputStream ouputStream = response.getOutputStream();
            wb.write(ouputStream);
            ouputStream.flush();
            ouputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
            log.error(String.format("{ExportToExcelUtil.export} ==> 导出Excel失败,%s", e.getMessage()));
        }
    }

    /**
     * 根据字段名获取字段对象
     *
     * @param fieldName 字段名
     * @param clazz     包含该字段的类
     * @return 字段
     */
    public static Field getFieldByName(String fieldName, Class<?> clazz) {
        // 拿到本类的所有字段
        Field[] selfFields = clazz.getDeclaredFields();

        // 如果本类中存在该字段，则返回
        for (Field field : selfFields) {
            //如果本类中存在该字段，则返回
            if (field.getName().equals(fieldName)) {
                return field;
            }
        }

        // 否则，查看父类中是否存在此字段，如果有则返回
        Class<?> superClazz = clazz.getSuperclass();
        if (superClazz != null && superClazz != Object.class) {
            //递归
            return getFieldByName(fieldName, superClazz);
        }

        // 如果本类和父类都没有，则返回空
        return null;
    }

    /**
     * 根据字段名获取字段值
     *
     * @param fieldName 字段名
     * @param o         对象
     * @return 字段值
     * @throws Exception 异常
     */
    public static Object getFieldValueByName(String fieldName, Object o)
            throws Exception {

        Object value = null;
        //根据字段名得到字段对象
        Field field = getFieldByName(fieldName, o.getClass());

        //如果该字段存在，则取出该字段的值
        if (field != null) {
            field.setAccessible(true);//类中的成员变量为private,在类外边使用属性值，故必须进行此操作
            value = field.get(o);//获取当前对象中当前Field的value
        } else {
            throw new Exception(o.getClass().getSimpleName() + "类不存在字段名 "
                    + fieldName);
        }

        return value;
    }

    /**
     * 根据带路径或不带路径的属性名获取属性值,即接受简单属性名，
     * 如userName等，又接受带路径的属性名，如student.department.name等
     *
     * @param fieldNameSequence 带路径的属性名或简单属性名
     * @param o                 对象
     * @return 属性值
     * @throws Exception 异常
     */
    public static Object getFieldValueByNameSequence(String fieldNameSequence, Object o) throws Exception {
        Object value = null;

        // 将fieldNameSequence进行拆分
        String[] attributes = fieldNameSequence.split("\\.");
        if (attributes.length == 1) {
            value = getFieldValueByName(fieldNameSequence, o);
        } else {
            // 根据数组中第一个连接属性名获取连接属性对象，如student.department.name
            Object fieldObj = getFieldValueByName(attributes[0], o);
            //截取除第一个属性名之后的路径
            String subFieldNameSequence = fieldNameSequence.substring(fieldNameSequence.indexOf(".") + 1);
            //递归得到最终的属性对象的值
            value = getFieldValueByNameSequence(subFieldNameSequence, fieldObj);
        }
        return value;

    }

    /**
     * 向工作表中填充数据
     *
     * @param sheet          excel的工作表名称
     * @param list           数据源
     * @param colHeader      表头
     * @param titleCellStyle 标题单元格样式
     * @throws Exception 异常
     */
    public static <T> void fillSheet(XSSFSheet sheet, List<String> list, List<String> colHeader, XSSFCellStyle titleCellStyle, XSSFCellStyle cellStyle) throws Exception {

        //在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
        XSSFRow row = sheet.createRow((int) 0);
        row.setHeight((short) 800);

        // 填充表头
        for (int i = 0; i < colHeader.size(); i++) {
            XSSFCell cell = row.createCell(i);
            cell.setCellValue(colHeader.get(i));
            sheet.setColumnWidth(i, 4000);
            cell.setCellStyle(titleCellStyle);
        }

        // 填充内容
        for (int index = 0; index < list.size(); index++) {
            row = sheet.createRow(index + 1);
            row.setHeight((short) 500);
            // 获取单个对象
            String item = list.get(index);
            String[] split = item.split(",");
            for (int i = 0; i < colHeader.size(); i++) {
                XSSFCell cell = row.createCell(i);
                cell.setCellStyle(cellStyle);

                cell.setCellValue(split[i] == null ? "" : split[i]);
            }
        }
    }

    /**
     * @param date date
     * @return String
     * @Title: getDateStr
     * @Description: 格式化日期格式 : xxxx年xx月xx日
     */
    public static String getDateStr(Date date) {
        return String.format("%1$tY/%1$tm/%1$td", date);
    }

    /**
     * 设置：Cell样式
     *
     * @param cellStyle         单元格样式
     * @param pattern           设置为 1 会用前景色填充单元格
     * @param backgroundColor   设置前景色填充颜色
     * @param alignment         设置单元格的水平对齐类型
     * @param verticalAlignment 设置单元格的垂直对齐类型
     */
    public static void writeCellStyle(CellStyle cellStyle, FillPatternType pattern, short backgroundColor,
                                      HorizontalAlignment alignment, VerticalAlignment verticalAlignment) {
        cellStyle.setFillPattern(pattern);
        cellStyle.setFillForegroundColor(backgroundColor);
        cellStyle.setAlignment(alignment);
        cellStyle.setVerticalAlignment(verticalAlignment);
    }

}
