package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.bo.EwbCalcParamBo;
import com.nationrel.modules.weibull.entity.EwbAnalysisSetting;
import com.nationrel.modules.weibull.enums.EwbConfidenceTypeCodeEnum;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.enums.EwbParameterCodeEnum;
import com.nationrel.modules.weibull.vo.EwbAnalysisStartVo;
import com.nationrel.modules.weibull.vo.EwbAnalysisStressStrengthVo;
import com.nationrel.modules.weibull.vo.EwbProbabilityPlotVo;
import com.nationrel.modules.weibull.vo.EwbReliabilityPlotVo;
import com.nationrel.modules.weibull.service.EwbCalcService;
import com.nationrel.modules.weibull.service.EwbPlotService;
import com.nationrel.modules.weibull.service.EwbValidatorService;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import jakarta.validation.constraints.NotBlank;
import java.math.BigDecimal;
import java.util.Map;

/**
 * @author Aimee on 2020/10/19.
 */
@Tag(name = "寿命数据分析-画图部分")
@Validated
@RestController
@RequestMapping("/weibull/chart")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbChartController {

    private final EwbPlotService ewbPlotService;
    private final EwbCalcService ewbCalcService;
    private final EwbValidatorService ewbValidatorService;

    @Operation(summary = "获取计算器设置")
    @GetMapping("/calc-setting")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "settingId", description = "设置ID", in = ParameterIn.QUERY)
    })
    public ServerResponse calcSetting(@NotBlank(message = "SetIdNotEmpty") String settingId, @NotBlank(message = "AnalysisIdNotEmpty") String analysisId) {
        Map<String, Object> resultMap = ewbCalcService.calcSetting(GlobalExceptionHandler.parseUserInfo().getUserId(), settingId, analysisId);
        return ServerResponse.createBySuccess(resultMap);
    }

    @Operation(summary = "更新画图的标记以及xy区间信息")
    @PostMapping("/markInfo")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse updateMarkInfo(@RequestBody EwbAnalysisSetting param) {
        return ewbPlotService.updateMarkInfo(param, GlobalExceptionHandler.parseUserInfo().getLanguageKey());
    }

    @Operation(summary = "更新分布概率图的图形的名称")
    @PutMapping("/reliabilityPlotName")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse updateReliabilityPlotName(@RequestBody EwbAnalysisSetting param) {
        return ewbPlotService.updateReliabilityPlotName(param);
    }

    @Operation(summary = "获取计算结果")
    @GetMapping("/calcResult")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "settingId", description = "设置ID", in = ParameterIn.QUERY)
    })
    public ServerResponse calcResult(@NotBlank(message = "AnalysisIdNotEmpty") String analysisId) {
        return ServerResponse.createBySuccess(ewbCalcService.calcResult(analysisId, GlobalExceptionHandler.parseUserInfo().getLanguageKey()));
    }

    @Operation(summary = "开始计算")
    @PostMapping("/calc")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse calc(@RequestBody EwbCalcParamBo param) throws Exception {
        HeaderUserInfo user = GlobalExceptionHandler.parseUserInfo();
        Integer languageKey = user.getLanguageKey();
        // 计算类型-可靠度
        String distributingCode = param.getDistributingCode();
        String parameterCode = param.getParameterCode();
        String gamma = param.getGamma();
        BigDecimal gammaBd = new BigDecimal(gamma);
        if (EwbConstants.CALC_TYPE_RELIABILITY.equals(param.getCalcType())) {
            // 结束时间校验--分布为Exponential 2p时结束时间应大于等于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex()) && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex()) && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), gammaBd, EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            } else {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-不可靠度
        } else if (EwbConstants.CALC_TYPE_UNRELIABILITY.equals(param.getCalcType())) {
            // 结束时间校验--分布为Exponential 2p时结束时间应大于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex())
                    && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex())
                    && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), gammaBd, EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            } else {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-平均失效时间
        } else if (EwbConstants.CALC_TYPE_MTBF.equals(param.getCalcType())) {
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-失效率
        } else if (EwbConstants.CALC_TYPE_FAILURERATE.equals(param.getCalcType())) {
            // 结束时间校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.结束时间, languageKey), param.getEndTime(), EwbConstants.CALC_RANGE_END_TIME_MIN, EwbConstants.CALC_RANGE_END_TIME_MAX, languageKey);
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-参数
        } else if (EwbConstants.CALC_TYPE_PARAMETER.equals(param.getCalcType())) {
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-BX%寿命
        } else if (EwbConstants.CALC_TYPE_BLIFE.equals(param.getCalcType())) {
            // 失效百分比校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.不可靠度, languageKey), param.getFailurePercent(), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-可靠寿命
        } else if (EwbConstants.CALC_TYPE_TIMEATRELIABILITY.equals(param.getCalcType())) {
            // 可靠寿命百分比校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.可靠度, languageKey), param.getFailurePercent(), EwbConstants.CALC_RANGE_FAILURE_PERCENT_MIN, EwbConstants.CALC_RANGE_FAILURE_PERCENT_MAX, languageKey);

            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 计算类型-条件可靠度
        } else if (EwbConstants.CALC_TYPE_CONDITIONALPROBABILITY.equals(param.getCalcType()) || EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(param.getCalcType())) {
            // 开始时间校验
            // Exponential 2p 开始时间应大于等于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex())
                    && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex())
                    && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), gammaBd, EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            } else {
                calcParamCheckZero(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 附加时间校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.附加时间, languageKey), param.getAdditionalTime(), EwbConstants.CALC_RANGE_TACK_TIME_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
            // 计算类型-条件不可靠度
        } else if (EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(param.getCalcType()) || EwbConstants.CALC_TYPE_CONDITIONALUNRELIABILITY.equals(param.getCalcType())) {
            // 开始时间校验
            // Exponential 2p 开始时间应大于等于gamma
            if (ValidateUtil.isNotEmpty(distributingCode) && distributingCode.equals(EwbDistributingCodeEnum.Exponential.getIndex())
                    && ValidateUtil.isNotEmpty(parameterCode) && parameterCode.equals(EwbParameterCodeEnum.二参数.getIndex())
                    && ValidateUtil.isNotEmpty(gamma)) {
                timeCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), gammaBd, EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            } else {
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.开始时间, languageKey), param.getStartTime(), EwbConstants.CALC_RANGE_START_TIME_MIN, EwbConstants.CALC_RANGE_START_TIME_MAX, languageKey);
            }
            // 根据置信类型校验
            if (ValidateUtil.isNotEmpty(param.getConfidenceType()) && !EwbConfidenceTypeCodeEnum.无.getIndex().equals(param.getConfidenceType())) {
                // 置信度非空
                calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.置信度, languageKey), param.getConfidencePercent(), EwbConstants.BELIEVEPERCENT_MIN, EwbConstants.BELIEVEPERCENT_MAX, languageKey);
            }
            // 附加时间校验
            calcParamCheck(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.附加时间, languageKey), param.getAdditionalTime(), EwbConstants.CALC_RANGE_TACK_TIME_MIN, EwbConstants.CALC_RANGE_TACK_TIME_MAX, languageKey);
        }
        return ewbCalcService.calc(user.getUserId(), param, user, false);
    }

    @Operation(summary = "概率图")
    @PostMapping("/probabilityPlot")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse probabilityPlot(@RequestBody @Valid EwbAnalysisStartVo param) throws Exception {
        EwbProbabilityPlotVo plotVo = ewbPlotService.probabilityPlot(GlobalExceptionHandler.parseUserInfo(), param, false);
        return ServerResponse.createBySuccess(plotVo);
    }

//    @Operation(summary = "概率图数据校验")
//    @PutMapping("/probabilityPlot")
//    @Parameters(value = {
//            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
//    })
//    public ServerResponse probabilityPlotAna(@RequestBody @Valid EwbAnalysisStartVo param) {
//        return ewbValidatorService.validatorDataDetail(param.getAnalysisId(), param.getDataId(), param.getSettingId(), param.getParameterCode(), param.getDistributingCode(), param.getEstimationMethodCode(), param.getBelievePercent(), GlobalExceptionHandler.parseHeader(request));
//    }

    @Operation(summary = "stressStrength曲线图")
    @PostMapping("/stressStrengthPlot")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse stressStrengthPlot(@RequestBody EwbAnalysisStressStrengthVo param) throws Exception {
        return ewbPlotService.stressStrengthPlot(param);
    }

    @Operation(summary = "除概率图外的其他图")
    @PostMapping("/image")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse otherPlot(@RequestBody @Valid EwbAnalysisStartVo param) throws Exception {
        EwbReliabilityPlotVo plotVo = ewbPlotService.selectEchartDetail(GlobalExceptionHandler.parseUserInfo(), param);
        return ServerResponse.createBySuccess(plotVo);
    }

    /**
     * @param fieldName  字段名
     * @param gamma      Exponential 2p gamma
     * @param fieldValue 字段值
     * @param min        最小值
     * @param max        最大值
     * @Title: timeCheck
     * @Description: 时间参数校验
     */
    private void timeCheck(String fieldName, String fieldValue, BigDecimal gamma, BigDecimal min, BigDecimal max, Integer languageKey) {
        // 不能为空
        if (ValidateUtil.isEmpty(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.不能为空, languageKey);
        } else if (!ValidateUtil.isNumber(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.必须为数字, languageKey);
        }
        // Exponential 2p可靠度结束时间、不可靠度结束时间、条件可靠度开始时间必须大于gamma
        BigDecimal val = WeibullNumberUtil.parseBigDecimal(fieldValue);
        if (ValidateUtil.isPositiveNumber(fieldValue)) {
            if (val.compareTo(gamma) <= 0 || val.compareTo(max) > 0) {
                throw new DescribeException(fieldName, gamma, max, EwbCustomPromptEnum.范围必须是, languageKey);
            }
        } else {
            // 范围
            if (val.compareTo(min) <= 0 || val.compareTo(max) > 0) {
                throw new DescribeException(fieldName, min, max, EwbCustomPromptEnum.范围必须是, languageKey);
            }

        }
    }

    /**
     * @param fieldName  待校验字段名
     * @param fieldValue 待校验字段值
     * @param min        校验范围最小值
     * @param max        校验范围最大值
     * @Title: calcParamCheck
     * @Description: 参数校验
     */
    public static void calcParamCheck(String fieldName, String fieldValue, BigDecimal min, BigDecimal max, Integer languageKey) {

        if (ValidateUtil.isEmpty(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.不能为空, languageKey);
        } else if (!ValidateUtil.isNumber(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.必须为数字, languageKey);
        } else if (WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(min) <= 0 || WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(max) >= 0) {
            throw new DescribeException(fieldName, min, max, EwbCustomPromptEnum.范围必须是, languageKey);
        }
    }

    /**
     * 可为0
     * @param fieldName  待校验字段名
     * @param fieldValue 待校验字段值
     * @param min        校验范围最小值
     * @param max        校验范围最大值
     * @Title: calcParamCheck
     * @Description: 参数校验
     */
    public static void calcParamCheckZero(String fieldName, String fieldValue, BigDecimal min, BigDecimal max, Integer languageKey) {

        if (ValidateUtil.isEmpty(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.不能为空, languageKey);
        } else if (!ValidateUtil.isNumber(fieldValue)) {
            throw new DescribeException(fieldName, EwbCustomPromptEnum.必须为数字, languageKey);
        } else if (WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(min) < 0 || WeibullNumberUtil.parseBigDecimal(fieldValue).compareTo(max) > 0) {
            throw new DescribeException(fieldName, min, max, EwbCustomPromptEnum.范围必须是, languageKey);
        }
    }

}
