package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.bo.EwbCalcRandomBo;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.vo.EwbAnalysisDataSaveVo;
import com.nationrel.modules.weibull.service.EwbAnalysisDataDetailService;
import com.nationrel.modules.weibull.service.EwbRandomService;
import com.nationrel.modules.weibull.util.WeibullNumberUtil;
import com.nationrel.modules.weibull.util.ValidateUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import java.io.IOException;
import java.math.BigDecimal;

/**
 * @author Aimee on 2020/10/19.
 */
@Tag(name = "寿命数据分析-数据部分")
@Slf4j
@RestController
@RequestMapping("/weibull/data")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbDataController {
    private final EwbAnalysisDataDetailService ewbAnalysisDataDetailService;
    private final EwbRandomService ewbRandomService;

    @Operation(summary = "获取数据")
    @GetMapping("/getData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "dataId", description = "数据ID", in = ParameterIn.QUERY)
    })
    public ServerResponse getAnalysisData(
            @RequestParam String dataId,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "1000000") int pageSize) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return ewbAnalysisDataDetailService.listByPage(userInfo, dataId, pageIndex, pageSize);
    }

    @Operation(summary = "保存数据")
    @PostMapping("/saveData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse saveData(@RequestBody @Valid EwbAnalysisDataSaveVo param) {
        return ewbAnalysisDataDetailService.saveData(GlobalExceptionHandler.parseUserInfo(), param, false);
    }

    @Operation(summary = "随机数生成(成功之后调用获取数据接口)")
    @PostMapping("/random")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse random(@RequestBody EwbCalcRandomBo param) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        randomCheck(param, userInfo.getLanguageKey());
        return ewbRandomService.random(userInfo, param);
    }

    @Operation(summary = "导入")
    @PostMapping("/importData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse importData(HttpServletRequest request,
                                     @RequestParam("file") MultipartFile file,
                                     @RequestParam("dataId") String dataId,
                                     @RequestParam("analysisId") String analysisId) throws Exception{
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        ewbAnalysisDataDetailService.importData(userInfo, file, dataId, analysisId);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "导出")
    @GetMapping("/exportData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public void exportData(HttpServletResponse response, @RequestParam("dataId") String dataId) throws IOException {
        ewbAnalysisDataDetailService.exportData(response, dataId);
    }

    @Operation(summary = "导入模板下载")
    @GetMapping("/import-template")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public void importTemplate(HttpServletResponse response) {
        ewbAnalysisDataDetailService.importTemplate(response);
    }

    private void randomCheck(EwbCalcRandomBo param, Integer languageKey) {
        // 数据验证
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(param.getDistributingCode())) {
            if (ValidateUtil.isEmpty(param.getEstimationMethodCode())) {
                throw new DescribeException(EwbCustomPromptEnum.参数不可为空, languageKey);
            }
            if (ValidateUtil.isEmpty(param.getBeta())) {
                throw new DescribeException("beta" , EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getBeta())) {
                throw new DescribeException("beta", EwbCustomPromptEnum.必须为数字, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getBeta()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getBeta()) > 50) {
                throw new DescribeException("beta", new BigDecimal("0.0001"),  new BigDecimal("50"), EwbCustomPromptEnum.范围必须是, languageKey);
            }
            if (ValidateUtil.isEmpty(param.getEta())) {
                throw new DescribeException("eta", EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getEta())) {
                throw new DescribeException("eta",EwbCustomPromptEnum.必须为数字, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getEta()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getEta()) > 1000000000) {
                throw new DescribeException("eta", new BigDecimal("0.0001"), new BigDecimal("1000000000"), EwbCustomPromptEnum.范围必须是, languageKey);
            }
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(param.getDistributingCode())) {
            if (ValidateUtil.isEmpty(param.getLogMean())) {
                throw new DescribeException("u'" , EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getLogMean())) {
                throw new DescribeException("u'", EwbCustomPromptEnum.必须为数字, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getLogMean()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getLogMean()) > 10) {
                throw new DescribeException("u'", new BigDecimal("0.0001"), new BigDecimal("10"), EwbCustomPromptEnum.范围必须是, languageKey);
            }
            if (ValidateUtil.isEmpty(param.getLogSigma())) {
                throw new DescribeException("σ'" , EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getLogSigma())) {
                throw new DescribeException("σ'", EwbCustomPromptEnum.必须为数字, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getLogSigma()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getLogSigma()) > 5) {
                throw new DescribeException("σ'", new BigDecimal("0.0001"), new BigDecimal("5"), EwbCustomPromptEnum.范围必须是, languageKey);
            }
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(param.getDistributingCode())) {
            if (ValidateUtil.isEmpty(param.getMean())) {
                throw new DescribeException("u" , EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getMean())) {
                throw new DescribeException("u", EwbCustomPromptEnum.必须为数字, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getMean()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getMean()) > 1000000000) {
                throw new DescribeException("u", new BigDecimal("0.0001"), new BigDecimal("1000000000"), EwbCustomPromptEnum.范围必须是, languageKey);
            }
            if (ValidateUtil.isEmpty(param.getSigma())) {
                throw new DescribeException("σ" , EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getSigma())) {
                throw new DescribeException("σ", EwbCustomPromptEnum.必须为数字, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getSigma()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getSigma()) > 1000000000) {
                throw new DescribeException("σ", new BigDecimal("0.0001"), new BigDecimal("1000000000"), EwbCustomPromptEnum.范围必须是, languageKey);
            }
        } else if (EwbDistributingCodeEnum.Exponential.getIndex().equals(param.getDistributingCode())) {
            if (ValidateUtil.isEmpty(param.getLamda())) {
                throw new DescribeException("λ" , EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getLamda())) {
                throw new DescribeException("λ", EwbCustomPromptEnum.必须为数字, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getLamda()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getLamda()) > 1000000000) {
                throw new DescribeException("λ", new BigDecimal("0.0001"), new BigDecimal("1000000000"), EwbCustomPromptEnum.范围必须是, languageKey);
            }
        }
        if (ValidateUtil.isEmpty(param.getProduceQuantity())) {
            throw new DescribeException(EwbCommonUsedEnum.生成点个数, EwbCustomPromptEnum.不可为空, languageKey);
        } else if (!ValidateUtil.isInteger(param.getProduceQuantity())) {
            throw new DescribeException(EwbCommonUsedEnum.生成点个数, EwbCustomPromptEnum.必须为整数, languageKey);
        } else if (WeibullNumberUtil.parseInteger(param.getProduceQuantity()) < 2 || WeibullNumberUtil.parseInteger(param.getProduceQuantity()) > 400000) {
            throw new DescribeException(EwbCommonUsedEnum.生成点个数, EwbCustomPromptEnum.必须为整数, "2-400000", languageKey);
        }
        if ("2".equals(param.getCensoring())) {
            if (ValidateUtil.isEmpty(param.getFailureQuantity())) {
                throw new DescribeException(EwbCommonUsedEnum.失效数, EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isInteger(param.getFailureQuantity())) {
                throw new DescribeException(EwbCommonUsedEnum.失效数, EwbCustomPromptEnum.必须为整数, languageKey);
            } else if (WeibullNumberUtil.parseInteger(param.getFailureQuantity()) < 1 || WeibullNumberUtil.parseInteger(param.getFailureQuantity()) > WeibullNumberUtil.parseInteger(param.getProduceQuantity())) {
                throw new DescribeException(EwbCommonUsedEnum.失效数, EwbCustomPromptEnum.必须为整数, "1-" + param.getProduceQuantity(), languageKey);
            }
        } else if ("3".equals(param.getCensoring())) {
            if (ValidateUtil.isEmpty(param.getTime())) {
                throw new DescribeException(EwbCommonUsedEnum.时间, EwbCustomPromptEnum.不可为空, languageKey);
            } else if (!ValidateUtil.isNumber(param.getTime())) {
                throw new DescribeException(EwbCommonUsedEnum.时间, EwbCustomPromptEnum.必须为整数, languageKey);
            } else if (WeibullNumberUtil.parseDouble(param.getTime()) < 0.0001 || WeibullNumberUtil.parseDouble(param.getTime()) > 1000000000) {
                throw new DescribeException(EwbCommonUsedEnum.时间, EwbCustomPromptEnum.必须为整数, "0.0001-1000000000", languageKey);
            }
        }
    }


}
