package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.entity.EwbMultiChart;
import com.nationrel.modules.weibull.vo.*;
import com.nationrel.modules.weibull.service.EwbMultiChartService;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import jakarta.validation.constraints.NotBlank;
import java.util.List;

@Tag(name = "寿命数据多图")
@RestController
@RequestMapping("/weibull/multichart")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbMultiChartController {

    private final EwbMultiChartService ewbMultiChartService;

    @Operation(summary = "多图创建")
    @PostMapping("/create")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse create(@RequestBody @Valid EwbMultiChartCreateVo param) {
        return ewbMultiChartService.createMultiChart(GlobalExceptionHandler.parseUserInfo(), param);
    }

    @Operation(summary = "更新画图的标记以及xy区间信息")
    @PostMapping("/markInfo")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse updateMarkInfo(@RequestBody EwbMultiChart param) {
        return ewbMultiChartService.updateMarkInfo(param, GlobalExceptionHandler.parseUserInfo().getLanguageKey());
    }

    @Operation(summary = "多图列表")
    @GetMapping("/list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse list(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return ServerResponse.createBySuccess(ewbMultiChartService.list(wbId));
    }

    @Operation(summary = "多图名称修改-删除(修改名称传多图名称，删除时del_flag=1)")
    @PostMapping("/edit")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse edit(@RequestBody EwbMultiChart ewbMultiChart) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        ewbMultiChartService.edit(userInfo, ewbMultiChart);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "分析列表")
    @GetMapping("/analysis-list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY),
            @Parameter(name = "status", description = "分析类型", in = ParameterIn.QUERY)
    })
    public ServerResponse analysisList(@RequestParam String wbId, @RequestParam(required = false) String multiChartId, @RequestParam(required = false) String analysisType) {
        if (StringUtils.isBlank(wbId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        List<EwbMultiChartAnalysisListVo> analysisListVos = ewbMultiChartService.selectAnalysisList(wbId, multiChartId, analysisType, userInfo.getUserId());
        return ServerResponse.createBySuccess(analysisListVos);
    }

    @Operation(summary = "分析保存")
    @PostMapping("/save-list")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveList(@RequestBody EwbMultiChartSaveListVo param) {
        if (StringUtils.isBlank(param.getMultiChartId())) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.多图ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        if (StringUtils.isBlank(param.getAnalysisIds())) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.至少选择一个分析不可以为空, GlobalExceptionHandler.parseLk());
        }
        ewbMultiChartService.saveList(param, GlobalExceptionHandler.parseLk());
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "概率图")
    @GetMapping("/probabilityPlot")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse probabilityPlot(@RequestParam String multiChartId) {
        if (StringUtils.isBlank(multiChartId)) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.多图ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        List<EwbProbabilityPlotVo> ewbProbabilityPlotVos = ewbMultiChartService.probabilityPlot(userInfo, multiChartId);
        return ServerResponse.createBySuccess(ewbProbabilityPlotVos);
    }

    @Operation(summary = "可靠度")
    @GetMapping("/image")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse reliabilityPlot(@RequestParam String multiChartId, @RequestParam String plotType, @RequestParam Double xaxisStart, @RequestParam Double xaxisEnd, @RequestParam Double yaxisStart, @RequestParam Double yaxisEnd) {
        if (StringUtils.isBlank(multiChartId)) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.多图ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        List<EwbProbabilityPlotVo> ewbProbabilityPlotVos = ewbMultiChartService.selectEchartDetail(userInfo, multiChartId, plotType, xaxisStart, xaxisEnd, yaxisStart, yaxisEnd);
        return ServerResponse.createBySuccess(ewbProbabilityPlotVos);
    }
}
