package com.nationrel.modules.weibull.controller;

import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.dto.EwbAnalysisNevadaPlotVo;
import com.nationrel.modules.weibull.dto.EwbNevadaSettingDto;
import com.nationrel.modules.weibull.entity.EwbNonNevadaSetting;
import com.nationrel.modules.weibull.vo.*;
import com.nationrel.modules.weibull.service.EwbNevadaService;
import com.nationrel.modules.weibull.service.EwbNonNevadaService;
import com.nationrel.modules.weibull.handler.GlobalExceptionHandler;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;

@Tag(name = "内华达分析（整合nevada和nonnevada）")
@Validated
@RestController
@RequestMapping("/weibull/nevada")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbNevadaController {

    private final EwbNevadaService nevadaService;
    private final EwbNonNevadaService nonNevadaService;

    @Operation(summary = "内华达创建（整合nevada和nonnevada）")
    @PostMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse create(@RequestBody @Valid NevadaCreateVo param) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return "nonnevada".equals(param.getWhichTree()) ? nonNevadaService.create(param, userInfo) : nevadaService.create(param, userInfo);
    }

    @Operation(summary = "内华达列表")
    @GetMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse list(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return nonNevadaService.list(wbId);
    }

    @Operation(summary = "内华达数据更新")
    @PutMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "wbId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse<NevadaDataVo> editNevadaData(@RequestBody NevadaSettingVo nevadaSetting) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return "nonnevada".equals(nevadaSetting.getWhichTree()) ? nonNevadaService.editNevadaData(nevadaSetting, false, userInfo) : nevadaService.editNevadaData(nevadaSetting, false, userInfo);
    }

    @Operation(summary = "内华达页面初始化获取数据（整合nevada和nonnevada）")
    @GetMapping("/init")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<NevadaDataVo> getNevadaData(@NotBlank(message = "AnalysisIdNotEmpty") String nevadaId, @RequestParam(value = "whichTree") String whichTree) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return "nonnevada".equals(whichTree) ? nonNevadaService.editNevadaData(NevadaSettingVo.builder().nevadaId(nevadaId).build(), true, userInfo) : nevadaService.editNevadaData(NevadaSettingVo.builder().nevadaId(nevadaId).build(), true, userInfo);
    }

    @Operation(summary = "内华达表格数据保存")
    @PutMapping("/table")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveNevadaTable(@RequestBody NevadaSaveVo nevadaData) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return "nonnevada".equals(nevadaData.getWhichTree()) ? nonNevadaService.saveNevadaTable(userInfo, nevadaData) : nevadaService.saveNevadaTable(userInfo, nevadaData);
    }

    @Operation(summary = "内华达表格数据保存")
    @PutMapping("/kpitable")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse saveKpiNevadaTable(@RequestBody NevadaKpiResultVo nevadaData) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return nevadaService.saveKpiNevadaTable(userInfo, nevadaData);
    }

    @Operation(summary = "内华达表格数据保存")
    @PutMapping("/validNevada")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "nevadaId", description = "项目ID", in = ParameterIn.QUERY)
    })
    public ServerResponse validNevada(@NotBlank(message = "AnalysisIdNotEmpty") String nevadaId, @NotNull(message = "DataNotEmpty") Integer action) {
        return nonNevadaService.validData(nevadaId, action, GlobalExceptionHandler.parseLk());
    }

    @Operation(summary = "获取内华达表格数据")
    @GetMapping("/table")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "laims per验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getNevadaTableData(@NotBlank(message = "AnalysisIdNotEmpty") String nevadaId, @RequestParam(value = "claimsPer", required = false) Integer claimsPer) {
        return nevadaService.getNevadaTableData(nevadaId, claimsPer);
    }

    @Operation(summary = "获取内华达表格数据")
    @GetMapping("/kpitable")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "laims per验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse getKpiNevadaTableData(@NotBlank(message = "AnalysisIdNotEmpty") String nevadaId) {
        return nevadaService.getKpiNevadaTableData(nevadaId);
    }

    @Operation(summary = "更新kpi内华达设置数据")
    @PutMapping("/kpiSetting")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveKpiNevadaSetting(@RequestBody NevadaKpiResultVo resultVo) {
        return nevadaService.saveKpiNevadaSetting(resultVo);
    }

    @Operation(summary = "内华达预测数据")
    @GetMapping("/forecast")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "nevadaId", description = "内华达id", in = ParameterIn.QUERY),
            @Parameter(name = "forecast", description = "预测数量", in = ParameterIn.QUERY),
            @Parameter(name = "forecastType", description = "预测方式", in = ParameterIn.QUERY),
    })
    public ServerResponse forecast(HttpServletRequest request,
                                   @NotBlank(message = "AnalysisIdNotEmpty") String nevadaId,
                                   @RequestParam(value = "whichTree") String whichTree,
                                   @RequestParam(value = "forecast", defaultValue = "0") Integer forecast,
                                   @RequestParam(value = "forecastType", defaultValue = "0") Integer forecastType,
                                   @RequestParam(value = "boundSided", defaultValue = "00") String boundSided,
                                   @RequestParam(value = "confidenceLevel", defaultValue = "90") Double confidenceLevel,
                                   @RequestParam(value = "confidenceBound", defaultValue = "0") Integer confidenceBound) throws Exception {
        return nonNevadaService.forecastData(whichTree, nevadaId, forecast, forecastType, boundSided, confidenceLevel, confidenceBound, GlobalExceptionHandler.parseUserInfo());
    }

    @Operation(summary = "内华达预测数据导出")
    @PostMapping("/forecastExport")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
    })
    public void forecastExport(HttpServletResponse response,
                               @RequestBody NevadaDataVo nevadaDataVo) {
        nonNevadaService.forecastExport(nevadaDataVo, response);
    }

    @Operation(summary = "删除内华达数据")
    @DeleteMapping("/{nevadaId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<NevadaDataVo> delNevadaData(@PathVariable(value = "nevadaId") String nevadaId) {
        return nonNevadaService.delNevadaData(nevadaId);
    }

    @Operation(summary = "内华达数据复制")
    @PostMapping("/copy")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "nevadaId", description = "内华达ID", in = ParameterIn.QUERY),
    })
    public ServerResponse copyNevada(@RequestBody @Valid EwbNonNevadaSetting nevadaSetting) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        nonNevadaService.copyNevada(userInfo, nevadaSetting.getNevadaId());
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "内华达图名称修改")
    @PutMapping("/name")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse editNevada(@RequestBody EwbNevadaSettingDto nevadaSetting) {
        return nonNevadaService.editNevada(nevadaSetting);
    }

    @Operation(summary = "概率图")
    @PostMapping("/probabilityPlot")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse probabilityPlot(@RequestBody @Valid EwbAnalysisNevadaPlotVo param) {
        HeaderUserInfo user = GlobalExceptionHandler.parseUserInfo();
        EwbProbabilityPlotVo plotVo = nevadaService.probabilityPlot(user, param);
        return ServerResponse.createBySuccess(plotVo);
    }
}
