package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.enums.EwbCalcParametersEnum;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.modules.weibull.general.exception.DescribeException;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.mapper.EwbAnalysisSettingMapper;
import com.nationrel.modules.weibull.mapper.EwbInspectIntervalMapper;
import com.nationrel.modules.weibull.mapper.EwbReplaceIntervalMapper;
import com.nationrel.modules.weibull.entity.EwbAnalysisSetting;
import com.nationrel.modules.weibull.entity.EwbInspectInterval;
import com.nationrel.modules.weibull.entity.EwbReplaceInterval;
import com.nationrel.modules.weibull.vo.EwbCostOptimizeCreateVo;
import com.nationrel.modules.weibull.util.ValidateUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @author Aimee on 2020/11/23.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbCostOptimizeService {

    private final StringRedisTemplate redisTemplate;
    private final EwbReplaceIntervalMapper replaceIntervalMapper;
    private final EwbInspectIntervalMapper inspectIntervalMapper;
    private final EwbAnalysisSettingMapper analysisSettingMapper;

    private final AllVariableConfig allVariableConfig;

    /**
     * 最优替换间隔创建
     *
     * @param param
     * @param userInfo
     * @return
     */
    public String createReplace(EwbCostOptimizeCreateVo param, HeaderUserInfo userInfo) {
        EwbReplaceInterval replaceInterval = EwbReplaceInterval.builder().replaceIntervalId(UUIDUtil.getUuid())
                .wbId(param.getWbId())
                .costOptimizeName(param.getName())
                .analysisId(param.getAnalysisId())
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .updateTime(new Date())
                .updateUserId(userInfo.getUserId())
                .versionNo(1)
                .delFlag("0").build();
        canCostAnalysis(param, userInfo.getLanguageKey());
        replaceIntervalMapper.insert(replaceInterval);
        return replaceInterval.getReplaceIntervalId();
    }

    /**
     * 最优检测间隔创建
     *
     * @param param
     * @param userInfo
     * @return
     */
    public String createInspect(EwbCostOptimizeCreateVo param, HeaderUserInfo userInfo) {
        EwbInspectInterval inspectInterval = EwbInspectInterval.builder().inspectIntervalId(UUIDUtil.getUuid())
                .wbId(param.getWbId())
                .costOptimizeName(param.getName())
                .analysisId(param.getAnalysisId())
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .updateTime(new Date())
                .updateUserId(userInfo.getUserId())
                .versionNo(1)
                .delFlag("0").build();
        canCostAnalysis(param, userInfo.getLanguageKey());
        inspectIntervalMapper.insert(inspectInterval);
        return inspectInterval.getInspectIntervalId();
    }

    /**
     * 判断当前选择的weibull数据是否可以进行成本优化分析
     *
     * @param param
     */
    public void canCostAnalysis(EwbCostOptimizeCreateVo param, Integer languageKey) {
        // redis存在则直接获取
        JSONObject plotParam;
        String plotStr = redisTemplate.opsForValue().get(param.getAnalysisId());
        if (ValidateUtil.isNotEmpty(plotStr)) {
            plotParam = JSONObject.parseObject(plotStr);
        } else {
            // redis无数据则从数据库取数据
            EwbAnalysisSetting analysisSetting = analysisSettingMapper.selectOne(new QueryWrapper<EwbAnalysisSetting>().lambda().select(EwbAnalysisSetting::getCalcResultAll).eq(EwbAnalysisSetting::getSettingId, param.getSettingId()));
            if (analysisSetting != null) {
                plotStr = analysisSetting.getCalcResultAll();
            }
            plotParam = ValidateUtil.isNotEmpty(plotStr) ? JSONObject.parseObject(plotStr) : null;
        }
        // 若了两边都没有数据，则该weibull数据未完成分析，返回提示信息
        if (plotParam == null) {
            throw new DescribeException(EwbCustomPromptEnum.当前数据所关联weibull未完成分析, languageKey);
        }

        if (Double.parseDouble(JSONObject.parseObject(plotParam.get(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.results)).toString()).get(EwbCalcParametersEnum.getParameter(allVariableConfig.getServerSystem(), EwbCalcParametersEnum.beta)).toString()) < 1) {
            throw new DescribeException(EwbCustomPromptEnum.当前选择的weibull分布的β参数, languageKey);
        }
    }

    /**
     * 获取成本优化数据
     *
     * @param wbId
     * @return
     */
    public ServerResponse selectReplace(String wbId) {
        return ServerResponse.createBySuccess(replaceIntervalMapper.selectReplace(wbId));
    }

    /**
     * 获取成本优化数据
     *
     * @param wbId
     * @return
     */
    public ServerResponse selectInspect(String wbId) {
        return ServerResponse.createBySuccess(inspectIntervalMapper.selectInspect(wbId));
    }

    /**
     * 删除成本优化数据
     *
     * @param replaceIntervalId
     * @return
     */
    public ServerResponse deleteReplace(String replaceIntervalId) {
        replaceIntervalMapper.deleteById(replaceIntervalId);
        return ServerResponse.createBySuccess();
    }

    /**
     * 获取单条成本优化数据
     *
     * @param replaceIntervalId
     * @return
     */
    public ServerResponse getReplace(String replaceIntervalId) {
        return ServerResponse.createBySuccess(replaceIntervalMapper.selectById(replaceIntervalId));
    }

    /**
     * 修改成本优化名称
     *
     * @param replaceInterval
     * @return
     */
    public ServerResponse editReplace(EwbReplaceInterval replaceInterval) {
        EwbReplaceInterval preReplaceInterval = replaceIntervalMapper.selectById(replaceInterval.getReplaceIntervalId());
        preReplaceInterval.setCostOptimizeName(replaceInterval.getCostOptimizeName());
        replaceIntervalMapper.updateById(preReplaceInterval);
        return ServerResponse.createBySuccess();
    }

    /**
     * 删除检测成本优化数据
     *
     * @param inspectIntervalId
     * @return
     */
    public ServerResponse deleteInspect(String inspectIntervalId) {
        inspectIntervalMapper.deleteById(inspectIntervalId);
        return ServerResponse.createBySuccess();
    }

    /**
     * 获取单条检测成本优化数据
     *
     * @param inspectIntervalId
     * @return
     */
    public ServerResponse getInspect(String inspectIntervalId) {
        return ServerResponse.createBySuccess(inspectIntervalMapper.selectById(inspectIntervalId));
    }

    /**
     * 修改检测、成本优化名称
     *
     * @param inspectInterval
     * @return
     */
    public ServerResponse editInspect(EwbInspectInterval inspectInterval) {
        EwbInspectInterval preInspectInterval = inspectIntervalMapper.selectById(inspectInterval.getInspectIntervalId());
        preInspectInterval.setCostOptimizeName(inspectInterval.getCostOptimizeName());
        inspectIntervalMapper.updateById(preInspectInterval);
        return ServerResponse.createBySuccess();
    }
}
