package com.nationrel.modules.weibull.service;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.nationrel.modules.weibull.config.AllVariableConfig;
import com.nationrel.modules.weibull.general.common.ServerResponse;
import com.nationrel.modules.weibull.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.weibull.general.util.common.UUIDUtil;
import com.nationrel.modules.weibull.general.domain.enums.common.EwbConstants;
import com.nationrel.modules.weibull.mapper.EwbStressStrengthMapper;
import com.nationrel.modules.weibull.entity.EwbAnalysisResult;
import com.nationrel.modules.weibull.entity.EwbStressStrength;
import com.nationrel.modules.weibull.enums.EwbDistributingCodeEnum;
import com.nationrel.modules.weibull.enums.EwbStressStrengthEnum;
import com.nationrel.modules.weibull.vo.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author hecs on 2022/11/23.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbStressStrengthService {

    private final EwbStressStrengthMapper stressStrengthMapper;
    private final RgaApiService rgaApiService;
    private final AllVariableConfig allVariableConfig;

    /**
     * 创建并初始化信息
     *
     * @param name
     * @param wbId
     * @param userInfo
     * @return
     */
    public String create(String name, String wbId, HeaderUserInfo userInfo) {
        String stressStrengthId = UUIDUtil.getUuid();
        EwbStressStrength stressStrength = EwbStressStrength.builder()
                .stressStrengthId(stressStrengthId)
                .wbId(wbId)
                .stressStrengthName(name)
                .distributingCode(EwbDistributingCodeEnum.Weibull.getIndex())
                .compareDistributingCode(EwbDistributingCodeEnum.Weibull.getIndex())
                .beta(3.0)
                .compareBeta(2.0)
                .eta(2000.0)
                .compareEta(2500.0)
                .logMiu(9.0)
                .compareLogMiu(8.0)
                .logSigma(8.0)
                .compareLogSigma(0.9)
                .miu(150.0)
                .compareMiu(140.0)
                .sigma(30.0)
                .compareSigma(40.0)
                .lambda(0.4)
                .compareLambda(0.5)
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .updateTime(new Date())
                .updateUserId(userInfo.getUserId())
                .versionNo(1).build();
        stressStrengthMapper.insert(stressStrength);
        return stressStrengthId;
    }

    /**
     * 查询树的列表
     *
     * @param wbId
     * @return
     */
    public ServerResponse<List<EwbStressStrengthVo>> listByPage(String wbId) {
        return ServerResponse.createBySuccess(stressStrengthMapper.selectVoByWbId(wbId));
    }

    /**
     * 获取单条设置信息
     *
     * @param id
     * @return
     */
    public ServerResponse getById(String id) {
        EwbStressStrength stressStrength = stressStrengthMapper.selectById(id);
        stressStrength.setInsertUserId(null);
        stressStrength.setUpdateUserId(null);
        return ServerResponse.createBySuccess(stressStrength);
    }

    /**
     * 查询关联weibull列表
     *
     * @param wbId
     * @param analysisId
     * @param distributingCode
     * @param type(stress:应力,strength:强度)
     * @return
     */
    public ServerResponse<List<EwbStressAnalysisListVo>> selectAnalysisList(String wbId, String analysisId, String distributingCode, String type) {
        List<EwbStressAnalysisListVo> list = stressStrengthMapper.selectAnalysisList(wbId, distributingCode);
        for (EwbStressAnalysisListVo vo : list) {
            if (StringUtils.isNotBlank(analysisId)) {
                vo.setCheckFlag(type.equals(EwbStressStrengthEnum.应力.getIndex()) && vo.getAnalysisId().equals(analysisId) ? EwbConstants.FLAG_CHECKED : EwbConstants.FLAG_NO_CHECKED);
            }
            StringBuilder calcResultsStr = new StringBuilder();
            EwbAnalysisDisWizardService.handleCalcResult(allVariableConfig.getServerSystem(), JSON.parseObject(vo.getAnalysisResult()), new EwbAnalysisResult(), rgaApiService, EwbCalcResultVo.builder().distributingCode(distributingCode).parameterCode(vo.getParameterCode()).build(), false).forEach(calcResult -> calcResultsStr.append(calcResult.getKey()).append("=").append(calcResult.getValue()).append("；"));
            vo.setAnalysisResult(calcResultsStr.toString());
        }
        return ServerResponse.createBySuccess(list);
    }

    /**
     * 更新ss关联分析
     *
     * @param entity
     * @return
     */
    public ServerResponse edit(EwbStressStrengthEditVo entity) {
        EwbStressStrength stressStrength = stressStrengthMapper.selectById(entity.getStressStrengthId());
        handleStressStrength(stressStrength, entity.getAnalysisResult().split("；"), entity.getDistributingCode(), entity.getType(), entity.getAnalysisId());
        stressStrengthMapper.updateById(stressStrength);
        stressStrength.setInsertUserId(null);
        stressStrength.setUpdateUserId(null);
        return ServerResponse.createBySuccess(stressStrength);
    }

    /**
     * 处理beta、eta、miu、sigma...
     *
     * @param stressStrength
     * @param analysisResults
     */
    private void handleStressStrength(EwbStressStrength stressStrength, String[] analysisResults, String distributingCode, String type, String analysisId) {
        double value1 = Double.parseDouble(analysisResults[0].split("=")[1]);
        double value2 = Double.parseDouble(analysisResults[1].split("=")[1]);
        if (type.equals(EwbStressStrengthEnum.应力.getIndex())) {
            stressStrength.setAnalysisId(analysisId);
            stressStrength.setDistributingCode(distributingCode);
        } else {
            stressStrength.setCompareDistributingCode(distributingCode);
        }
        if (EwbDistributingCodeEnum.Weibull.getIndex().equals(distributingCode)) {
            if (type.equals(EwbStressStrengthEnum.应力.getIndex())) {
                stressStrength.setBeta(value1);
                stressStrength.setEta(value2);
            } else {
                stressStrength.setCompareBeta(value1);
                stressStrength.setCompareEta(value2);
            }
        } else if (EwbDistributingCodeEnum.Lognormal.getIndex().equals(distributingCode)) {
            if (type.equals(EwbStressStrengthEnum.应力.getIndex())) {
                stressStrength.setLogMiu(value1);
                stressStrength.setLogSigma(value2);
            } else {
                stressStrength.setCompareLogMiu(value1);
                stressStrength.setCompareLogSigma(value2);
            }
        } else if (EwbDistributingCodeEnum.Normal.getIndex().equals(distributingCode)) {
            if (type.equals(EwbStressStrengthEnum.应力.getIndex())) {
                stressStrength.setMiu(value1);
                stressStrength.setSigma(value2);
            } else {
                stressStrength.setCompareMiu(value1);
                stressStrength.setCompareSigma(value2);
            }
        } else {
            if (type.equals(EwbStressStrengthEnum.应力.getIndex())) {
                stressStrength.setLambda(value1);
            } else {
                stressStrength.setCompareLambda(value1);
            }
        }
    }

    /**
     * 删除信息
     *
     * @param id
     * @return
     */
    public ServerResponse deleteById(String id) {
        stressStrengthMapper.deleteById(id);
        return ServerResponse.createBySuccess();
    }

    /**
     * 修改分析名称
     *
     * @param entity
     * @return
     */
    public ServerResponse editName(EwbStressStrengthEditNameVo entity) {
        stressStrengthMapper.update(null, new UpdateWrapper<EwbStressStrength>().lambda().eq(EwbStressStrength::getStressStrengthId, entity.getStressStrengthId()).set(EwbStressStrength::getStressStrengthName, entity.getAnalysisName()));
        return ServerResponse.createBySuccess();
    }
}
